%{
/********************************************/
/*          Ada to C++ translator           */
/*  (c) Oct-Nov 1995, Zoran Constantinescu  */
/*       UPB Computer Science  345CB        */
/* ada2cScanner.y  scanner description file */
/********************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ada2cSimb.h"
#include "ada2cParser.h"

int line_no=1;

int yyparse();
%}


BLANK		[ \t]
LIT_MICA	[a-z]
LIT_MARE	[A-Z]
CIFRA		[0-9]
CHAR_SPEC_BAZA	[\#\%\&\'\(\)\*\+\-\,\.\/\:\;\<\>\=\_\|]|\"\"
CHAR_SPEC	\$\?\@\[\]\\\{\}\~]\t\n
CHAR_BAZA	{LIT_MARE}|{CIFRA}|{CHAR_SPEC_BAZA}
CHAR		{CHAR_BAZA}|{LIT_MICA}|{CHAR_SPEC}

LITERA		{LIT_MICA}|{LIT_MARE}
LIT_CIFRA	{LITERA}|{CIFRA}
IDENTIF		{LITERA}(_?{LIT_CIFRA})*

LIT_CHAR	\'{CHAR}\'
SIR_CHAR	\"{CHAR}*\"

INTREG		{CIFRA}(_?{CIFRA})*
EXPONENT	[Ee](\+?|\-){INTREG}
BAZA		[23456789]|10|11|12|13|14|15|16
CIFRA_EXT	{CIFRA}|{LITERA}
INT_BAZAT	{CIFRA_EXT}(_|{CIFRA_EXT})*
NR_BAZAT	{BAZA}\#{INT_BAZAT}(\.{INT_BAZAT})?\#{EXPONENT}?
NR_ZEC		{INTREG}(\.{INTREG})?{EXPONENT}?
LIT_NUM		{NR_ZEC}|{NR_BAZAT}

%%

--.*$		{ yylval.str=(char *)malloc(strlen(yytext)-1);
		  strcpy(yylval.str,yytext+2);
                  return T_COMMENT; }
{BLANK}+				;
\n					line_no++;

[aA][nN][dD]				return T_AND;
[oO][rR]				return T_OR;
[xX][oO][rR]				return T_XOR;
[nN][oO][tT]			 	return T_NOT;
[aA][bB][sS]				return T_ABS;
[mM][oO][dD]				return T_MOD;
[rR][eE][mM]				return T_REM;

[\_\(\)\:\,\|]				return yytext[0];
\<\<					return T_ET_1;
\>\>					return T_ET_2;
\.\.					return T_SEP_DOM;

\=					return T_EQ;
\/\=					return T_NEQ;
\<					return T_LT;
\<\=					return T_LE;
\>					return T_GT;
\>\=					return T_GE;
\+					return T_ADD;
\-					return T_SUB;
\*					return T_MUL;
\/					return T_DIV;
\*\*					return T_EXP;
\:\=					return T_ATTRIB;
\;					return T_TERM;
\=\>					return T_REZ;

[iI][nN][tT][eE][gG][eE][rR]		return T_INTEGER;
[cC][hH][aA][rR][aA][cC][tT][eE][rR]	return T_CHARACTER;
[bB][oO][oO][lL][eE][aA][nN]		return T_BOOLEAN;
{LIT_CHAR}				return T_LIT_CHAR;
[fF][aA][lL][sS][eE]			return T_FALSE;
[tT][rR][uU][eE]			return T_TRUE;

[iI][fF]				return T_IF;
[tT][hH][eE][nN]			return T_THEN;
[eE][lL][sS][eE]			return T_ELSE;
[eE][lL][sS][iI][fF]			return T_ELSIF;

[bB][eE][gG][iI][nN]			return T_BEGIN;
[eE][nN][dD]				return T_END;
[nN][uU][lL][lL]			return T_NULL;

[tT][yY][pP][eE]			return T_TYPE;
[sS][uU][bB][tT][yY][pP][eE]            return T_SUBTYPE;
[cC][oO][nN][sS][tT][aA][nN][tT]	return T_CONSTANT;
[iI][sS]				return T_IS;
[rR][aA][nN][gG][eE]			return T_RANGE;
[nN][eE][wW]				return T_NEW;
[oO][fF]				return T_OF;
[aA][rR][rR][aA][yY]			return T_ARRAY;

[oO][tT][hH][eE][rR][sS]		return T_OTHERS;
[rR][eE][cC][oO][rR][dD]		return T_RECORD;
[cC][aA][sS][eE]			return T_CASE;
[wW][hH][eE][nN]			return T_WHEN;
[aA][cC][cC][eE][sS][sS]		return T_ACCESS;
[iI][nN]				return T_IN;
[lL][oO][oO][pP]			return T_LOOP;
[wW][hH][iI][lL][eE]			return T_WHILE;
[fF][oO][rR]				return T_FOR;
[rR][eE][vV][eE][rR][sS][eE]		return T_REVERSE;
[dD][eE][cC][lL][aA][rR][eE]		return T_DECLARE;
[eE][xX][cC][eE][pP][tT][iI][oO][nN]	return T_EXCEPTION;
[eE][xX][iI][tT]			return T_EXIT;
[gG][oO][tT][oO]			return T_GOTO;
[rR][eE][tT][uU][rR][nN]		return T_RETURN;
[oO][uU][tT]				return T_OUT;
[pP][rR][oO][cC][eE][dD][uU][rR][eE]	return T_PROCEDURE;
[fF][uU][nN][cC][tT][iI][oO][nN]	return T_FUNCTION;
[pP][aA][cC][kK][aA][gG][eE]		return T_PACKAGE;
[pP][rR][iI][vV][aA][tT][eE]		return T_PRIVATE;
[bB][oO][dD][yY]			return T_BODY;
[lL][iI][mM][iI][tT][eE][dD]		return T_LIMITED;
[uU][sS][eE]				return T_USE;
[tT][aA][sS][kK]			return T_TASK;
[eE][nN][tT][rR][yY]			return T_ENTRY;
[aA][cC][cC][eE][pP][tT]		return T_ACCEPT;
[dD][eE][lL][aA][yY]			return T_DELAY;
[sS][eE][lL][eE][cC][tT]		return T_SELECT;
[aA][bB][oO][rR][tT]			return T_ABORT;
[tT][eE][rR][mM][iI][nN][aA][tT][eE]	return T_TERMINATE;
[rR][aA][iI][sS][eE]			return T_RAISE;
[gG][eE][nN][eE][rR][iI][cC]		return T_GENERIC;
[wW][iI][tT][hH]			return T_WITH;
[dD][iI][gG][iI][tT][sS]		return T_DIGITS;
[dD][eE][lL][tT][aA]			return T_DELTA;
[sS][eE][pP][aA][rR][aA][tT][eE]	return T_SEPARATE;
[rR][eE][nN][aA][mM][eE][sS]		return T_RENAMES;
[pP][rR][aA][gG][mM][aA]		return T_PRAGMA;
[aA][tT]				return T_AT;

{LIT_NUM}	{ /* printf("\nLiteral num.:   %s\n",yytext); */
		  yylval.t.str=(char *)malloc(strlen(yytext)+1);
		  yylval.t.tip=(strchr(yytext,'.'))?2:0;
		  strcpy(yylval.t.str,yytext);
		  return T_LIT_NUM; }
{SIR_CHAR}	{ printf("\nSir caractere:  %s\n",yytext); 
                  return T_SIR_CHAR; }
{IDENTIF}	{ yylval.str=(char *)malloc(strlen(yytext)+1);
		  strcpy(yylval.str,yytext);
                  return T_IDENTIF; }



%%

int yywrap()
{
  return 1;
}

int yyerror(char *s)
{
  printf("ada2c:%d: %s\n",line_no,s);
  list_simb("zx");

  exit(1);
}

int main()
{

  init_simb();
  yyparse();
  list_simb("zx");

  return 0;
}

